
local json = require("json")
local http = require("resty.http")
local utils = require("utils")
local const = require("const")

local request_method = ngx.var.request_method

utils.check_login()
utils.check_permission("ComputeManage")

if (request_method == "GET") then
    local args = ngx.req.get_uri_args()
    if (nil == args) then
        utils.error_exit(400, "invalid parameter")
    end

    local job_id = args["JobId"]
    if (nil == job_id) then
        utils.error_exit(400, "invalid parameter")
    end

    local url_job = const.kapacitor.base_url .. '/v1/nova/jobs/' .. tostring(job_id)
    local httpc = http.new()
    local res, err = httpc:request_uri(url_job, {method = "GET"})
    if not res then
        utils.log_error("get kapacitor job detail failed:" .. err)
        utils.error_exit(500, "get kapacitor job detail failed")
    end

    local ok, job = pcall(json.decode, res.body)
    if (not ok) then
        utils.log_error("data from kapacitor is invalid: " .. res.body)
        utils.error_exit(500, "data from kapacitor is invalid")
    end

    local res = {}
    res["Job"] = json.encode(job["job"]["dsl"]) --to string
    res["Status"] = job["job"]["status"]
    res["Mode"] = job["job"]["mode"]
    res_raw = json.encode(res)
    utils.error_exit(200, res_raw)

elseif (request_method == "POST") then
    ngx.req.read_body()
    local args_json = ngx.req.get_body_data()
	local ok, args = pcall(json.decode, args_json)
    if (not(ok and args)) then
        utils.error_exit(400, "invalid parameter")
    end

    local job_id = args["JobId"]
    local mode = args["Mode"]
    local param = args["Param"]
    if (not (job_id and ("" ~= job_id) and param and mode and ("debug" == mode or "production" == mode))) then
        utils.error_exit(400, "invalid parameter")
    end

    local httpc = http.new()
    local url_job = const.kapacitor.base_url .. '/v1/nova/jobs/' .. tostring(job_id) 


    -- set job mode
    -- curl --request PATCH http://localhost:9092/kapacitor/v1/nova/jobs/1 --data '{"mode": "debug"}'
    local data = '{"mode": "' .. mode ..'"}' 
    print(url_job, data)
    local res, err = httpc:request_uri(url_job, { method = "PATCH", body = data })
    if (not (res and 200 == res.status)) then
        utils.log_error("set kapacitor job mode failed:" .. res.status)
        utils.error_exit(500, "set kapacitor job mode failed")
    end

    
    -- set replay data
    -- curl --request PUT http://localhost:9092/kapacitor/v1/nova/jobs/1/replays --data '[{"productkey":"xxxx"}]'
    local url_replay = url_job .. '/replays'
    local res, err = httpc:request_uri(url_replay, { method = "PUT", body = param })
    if (not (res and 200 == res.status))  then
        if (400 == res.status) then
            utils.log_warn("job parameter is invalid" .. param)
            utils.error_exit(400,  "job parameter is invalid")
        end
        utils.log_error("set kapacitor job parameter failed:" .. res.status)
        utils.error_exit(500, "set kapacitor job parameter failed")
    end
   
    local ok, repaly_info = pcall(json.decode, res.body)
    if (not ok) then
        utils.log_error("data from kapacitor is invalid: " .. res.body)
        utils.error_exit(500, "data from kapacitor is invalid")
    end

    -- { "replayid": "1-10", "status": "running" }
    local replayid = repaly_info["replayid"]
    if (not replayid) then
       utils.log_error("set kapacitor job parameter failed: " .. repaly_info["error"])
       utils.error_exit(400, "invalid kapacitor job parameter")
    end


    -- get replay status
    -- curl -i 'http://localhost:9092/kapacitor/v1/nova/jobs/1/replays/1-8' 
    local url_replay_status = url_replay .. '/' .. replayid
    local cnt = 0
    repeat 
        ngx.sleep(0.5)
        local res, err = httpc:request_uri(url_replay_status, { method = "GET" })
        if (not (res and 200 == res.status))  then
            utils.log_error("get replay status failed:" .. err)
            utils.error_exit(500, "get replay status failed")
        end

        local ok, status = pcall(json.decode, res.body)
        if (not ok) then
            utils.log_error("data from kapacitor is invalid: " .. res.body)
            utils.error_exit(500, "data from kapacitor is invalid")
        end

        cnt = cnt + 1
        if (cnt > 60) then
            utils.log_error("wait job end timeout")
            utils.error_exit(500, "wait job end timeout")
        end
        print(status["status"])
    until ("stopped" == status["status"])


    -- get replay log
    -- curl -i 'http://localhost:9092/kapacitor/v1/nova/jobs/1/replays/1-8/logs'
    local url_replay_log = url_replay_status .. '/' .. 'logs' 
    local res, err = httpc:request_uri(url_replay_log, { method = "GET" })
    if (not (res and 200 == res.status))  then
        utils.log_error("get replay log failed:" .. res.status)
        utils.error_exit(500, "get replay log failed")
    end

	local res_api = {}
    if (res.has_body) then
        local total_log = res.body
        repeat
            local body, err = res:read_body()
            if (body) then
                total_log = total_log .. body    
            end
        until (not body)
        res_api["Log"] = total_log
    else
        res_api["Log"] = "Not Log Content"
    end
    utils.error_exit(200, json.encode(res_api))
end

utils.error_exit(400, "unsupported request method")
------ EOF ------
